@@ -1,8 +1,8 @@
-/* $Id: nano.c,v 1.9 2003/10/03 20:26:25 dolorous Exp $ */
+/* $Id: nano.c,v 1.18 2004/01/09 23:04:55 dolorous Exp $ */
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
- *   Copyright (C) 1999-2003 Chris Allegretta                             *
+ *   Copyright (C) 1999-2004 Chris Allegretta                             *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -186,7 +186,7 @@ void print_view_warning(void)
     statusbar(_("Key illegal in VIEW mode"));
 }
 
-/* Initialize global variables - no better way for now.  If
+/* Initialize global variables -- no better way for now.  If
  * save_cutbuffer is nonzero, don't set cutbuffer to NULL. */
 void global_init(int save_cutbuffer)
 {
@@ -403,43 +403,43 @@ void help_init(void)
 
     /* Now add our shortcut info */
     for (s = currshortcut; s != NULL; s = s->next) {
-	/* true if the character in s->altval is shown in first column */
+	/* true if the character in s->metaval is shown in first column */
 	int meta_shortcut = 0;
 
-	if (s->val > 0 && s->val < 32)
-	    ptr += sprintf(ptr, "^%c", s->val + 64);
+	if (s->val != NANO_NO_KEY) {
 #ifndef NANO_SMALL
-	else if (s->val == NANO_CONTROL_SPACE)
-	    ptr += sprintf(ptr, "^%.6s", _("Space"));
-	else if (s->altval == NANO_ALT_SPACE) {
-	    meta_shortcut = 1;
-	    ptr += sprintf(ptr, "M-%.5s", _("Space"));
-	} else if (s->val == KEY_UP)
-	    ptr += sprintf(ptr, "%.2s", _("Up"));
+	    if (s->val == NANO_HISTORY_KEY)
+		ptr += sprintf(ptr, "%.2s", _("Up"));
+	    else
 #endif
-	else if (s->altval > 0) {
-	    meta_shortcut = 1;
-	    ptr += sprintf(ptr, "M-%c", s->altval -
-			(('A' <= s->altval && s->altval <= 'Z') ||
-			'a' <= s->altval ? 32 : 0));
+	    if (s->val == NANO_CONTROL_SPACE)
+		ptr += sprintf(ptr, "^%.5s", _("Space"));
+	    else if (s->val == NANO_CONTROL_8)
+		ptr += sprintf(ptr, "^?");
+	    else
+		ptr += sprintf(ptr, "^%c", s->val + 64);
 	}
-	/* Hack */
-	else if (s->val >= 'a') {
+#ifndef NANO_SMALL
+	else if (s->metaval != NANO_NO_KEY) {
 	    meta_shortcut = 1;
-	    ptr += sprintf(ptr, "M-%c", s->val - 32);
+	    if (s->metaval == NANO_ALT_SPACE)
+		ptr += snprintf(ptr, 8, "M-%.5s", _("Space"));
+	    else
+		ptr += sprintf(ptr, "M-%c", toupper(s->metaval));
 	}
+#endif
 
 	*(ptr++) = '\t';
 
-	if (s->misc1 > KEY_F0 && s->misc1 <= KEY_F(64))
-	    ptr += sprintf(ptr, "(F%d)", s->misc1 - KEY_F0);
+	if (s->func_key != NANO_NO_KEY)
+	    ptr += sprintf(ptr, "(F%d)", s->func_key - KEY_F0);
 
 	*(ptr++) = '\t';
 
-	if (!meta_shortcut && s->altval > 0)
-	    ptr += sprintf(ptr, "(M-%c)", s->altval -
-		(('A' <= s->altval && s->altval <= 'Z') || 'a' <= s->altval
-			? 32 : 0));
+	if (!meta_shortcut && s->metaval != NANO_NO_KEY)
+	    ptr += sprintf(ptr, "(M-%c)", toupper(s->metaval));
+	else if (meta_shortcut && s->misc != NANO_NO_KEY)
+	    ptr += sprintf(ptr, "(M-%c)", toupper(s->misc));
 
 	*(ptr++) = '\t';
 
@@ -452,7 +452,7 @@ void help_init(void)
     if (currshortcut == main_list)
 	for (t = toggles; t != NULL; t = t->next) {
 	    assert(t->desc != NULL);
-	    ptr += sprintf(ptr, "M-%c\t\t\t%s %s\n", t->val - 32, t->desc,
+	    ptr += sprintf(ptr, "M-%c\t\t\t%s %s\n", toupper(t->val), t->desc,
 				_("enable/disable"));
 	}
 #endif /* !NANO_SMALL */
@@ -659,7 +659,7 @@ void usage(void)
 #endif
     print1opt("-c", "--const", _("Constantly show cursor position"));
 #ifndef NANO_SMALL
-    print1opt("-d", "--rebinddelete", _("Fix Backspace if it acts like Delete"));
+    print1opt("-d", "--rebinddelete", _("Fix Backspace/Delete confusion problem"));
     print1opt("-i", "--autoindent", _("Automatically indent new lines"));
     print1opt("-k", "--cut", _("Let ^K cut from cursor to end of line"));
 #endif
@@ -774,7 +774,7 @@ void nano_disabled_msg(void)
 #endif
 
 #ifndef NANO_SMALL
-static int pid;		/* This is the PID of the newly forked process 
+static int pid;		/* This is the PID of the newly forked process
 			 * below.  It must be global since the signal
 			 * handler needs it. */
 RETSIGTYPE cancel_fork(int signal)
@@ -890,87 +890,56 @@ int open_pipe(const char *command)
 #ifndef DISABLE_MOUSE
 void do_mouse(void)
 {
-    MEVENT mevent;
-    int currslen;
-    const shortcut *s = currshortcut;
+    int mouse_x, mouse_y;
 
-    if (getmouse(&mevent) == ERR)
-	return;
+    if (get_mouseinput(&mouse_x, &mouse_y, 1) == 0) {
 
-    /* If mouse not in edit or bottom window, return */
-    if (wenclose(edit, mevent.y, mevent.x) && currshortcut == main_list) {
-	int sameline;
-	    /* Did they click on the line with the cursor?  If they
-	       clicked on the cursor, we set the mark. */
-	size_t xcur;
-	    /* The character they clicked on. */
+	/* Click in the edit window to move the cursor, but only when
+	   we're not in a subfunction. */
+	if (wenclose(edit, mouse_y, mouse_x) && currshortcut == main_list) {
+	    int sameline;
+		/* Did they click on the line with the cursor?  If they
+		   clicked on the cursor, we set the mark. */
+	    size_t xcur;
+		/* The character they clicked on. */
 
-	/* Subtract out size of topwin.  Perhaps we need a constant
-	 * somewhere? */
-	mevent.y -= 2;
+	    /* Subtract out size of topwin.  Perhaps we need a constant
+	       somewhere? */
+	    mouse_y -= 2;
 
-	sameline = mevent.y == current_y;
+	    sameline = (mouse_y == current_y);
 
-	/* Move to where the click occurred. */
-	for(; current_y < mevent.y && current->next != NULL; current_y++)
-	    current = current->next;
-	for(; current_y > mevent.y && current->prev != NULL; current_y--)
-	    current = current->prev;
+	    /* Move to where the click occurred. */
+	    for (; current_y < mouse_y && current->next != NULL; current_y++)
+		current = current->next;
+	    for (; current_y > mouse_y && current->prev != NULL; current_y--)
+		current = current->prev;
 
-	xcur = actual_x(current->data, get_page_start(xplustabs()) + mevent.x);
+	    xcur = actual_x(current->data, get_page_start(xplustabs()) +
+		mouse_x);
 
-	/* Selecting where the cursor is toggles the mark.  As does
-	   selecting beyond the line length with the cursor at the end of
-	   the line. */
-	if (sameline && xcur == current_x) {
-	    if (ISSET(VIEW_MODE)) {
-		print_view_warning();
-		return;
+	    /* Selecting where the cursor is toggles the mark.  As does
+	       selecting beyond the line length with the cursor at the
+	       end of the line. */
+	    if (sameline && xcur == current_x) {
+		if (ISSET(VIEW_MODE)) {
+		    print_view_warning();
+		    return;
+		}
+		do_mark();
 	    }
-	    do_mark();
+
+	    current_x = xcur;
+	    placewewant = xplustabs();
+	    edit_refresh();
 	}
-
-	current_x = xcur;
-	placewewant = xplustabs();
-	edit_refresh();
-    } else if (wenclose(bottomwin, mevent.y, mevent.x) && !ISSET(NO_HELP)) {
-	int i, k;
-
-	if (currshortcut == main_list)
-	    currslen = MAIN_VISIBLE;
-	else
-	    currslen = length_of_list(currshortcut);
-
-	if (currslen < 2)
-	    k = COLS / 6;
-	else 
-	    k = COLS / ((currslen + (currslen %2)) / 2);
-
-	/* Determine what shortcut list was clicked */
-	mevent.y -= (editwinrows + 3);
-
-	if (mevent.y < 0) /* They clicked on the statusbar */
-	    return;
-
-	/* Don't select stuff beyond list length */
-	if (mevent.x / k >= currslen)	
-	    return;
-
-	for (i = 0; i < (mevent.x / k) * 2 + mevent.y; i++)
-	    s = s->next;
-
-	/* And ungetch that value */
-	ungetch(s->val);
-
-	/* And if it's an alt-key sequence, we should probably send alt
-	   too ;-) */
-	if (s->val >= 'a' && s->val <= 'z')
-	   ungetch(27);
     }
+    /* FIXME: If we clicked on a location in the statusbar, the cursor
+       should move to the location we clicked on. */
 }
 #endif
 
-/* The user typed a printable character; add it to the edit buffer. */
+/* The user typed a character; add it to the edit buffer. */
 void do_char(char ch)
 {
     size_t current_len = strlen(current->data);
@@ -980,25 +949,33 @@ void do_char(char ch)
 	 * update_line()? */
 #endif
 
-    /* magic-line: when a character is inserted on the current magic line,
-     * it means we need a new one! */
-    if (filebot == current && current->data[0] == '\0') {
+    if (ch == '\0')		/* Null to newline, if needed. */
+	ch = '\n';
+    else if (ch == '\n') {	/* Newline to Enter, if needed. */
+	do_enter();
+	return;
+    }
+
+    assert(current != NULL && current->data != NULL);
+
+    /* When a character is inserted on the current magicline, it means
+     * we need a new one! */
+    if (filebot == current) {
 	new_magicline();
 	fix_editbot();
     }
 
-    /* more dangerousness fun =) */
+    /* More dangerousness fun =) */
     current->data = charealloc(current->data, current_len + 2);
     assert(current_x <= current_len);
-    charmove(&current->data[current_x + 1],
-	    &current->data[current_x],
-	    current_len - current_x + 1);
+    charmove(&current->data[current_x + 1], &current->data[current_x],
+	current_len - current_x + 1);
     current->data[current_x] = ch;
     totsize++;
     set_modified();
 
 #ifndef NANO_SMALL
-    /* note that current_x has not yet been incremented */
+    /* Note that current_x has not yet been incremented. */
     if (current == mark_beginbuf && current_x < mark_beginx)
 	mark_beginx++;
 #endif
@@ -1020,6 +997,48 @@ void do_char(char ch)
 #endif
 }
 
+int do_verbatim_input(void)
+{
+    char *verbatim_kbinput;	/* Used to hold verbatim input */
+    int verbatim_len;		/* Length of verbatim input */
+    int old_preserve = ISSET(PRESERVE), old_suspend = ISSET(SUSPEND);
+    int i;
+
+    /* Turn off Ctrl-Q (XON), Ctrl-S (XOFF), and Ctrl-Z (suspend) if
+     * they're on, so that we can use them to insert ^Q, ^S, and ^Z
+     * verbatim. */
+    if (old_preserve)
+	UNSET(PRESERVE);
+    if (old_suspend)
+	UNSET(SUSPEND);
+    if (old_preserve || old_suspend)
+	signal_init();
+
+    statusbar(_("Verbatim input"));
+    verbatim_kbinput = get_verbatim_kbinput(edit, &verbatim_len, 1);
+
+    /* Turn on DISABLE_CURPOS while inserting character(s) and turn it
+     * off afterwards, so that if constant cursor position display is
+     * on, it will be updated properly. */
+    SET(DISABLE_CURPOS);
+    for (i = 0; i < verbatim_len; i++)
+	do_char(verbatim_kbinput[i]);
+    UNSET(DISABLE_CURPOS);
+
+    free(verbatim_kbinput);
+
+    /* Turn Ctrl-Q, Ctrl-S, and Ctrl-Z back on if they were on
+     * before. */
+    if (old_preserve)
+	SET(PRESERVE);
+    if (old_suspend)
+	SET(SUSPEND);
+    if (old_preserve || old_suspend)
+	signal_init();
+
+    return 1;
+}
+
 int do_backspace(void)
 {
     if (current != fileage || current_x > 0) {
@@ -1361,7 +1380,7 @@ int do_wrap(filestruct *inptr)
 	i = indent_length(inptr->data);
 #endif
     wrap_line = inptr->data + i;
-    for(; i < len; i++, wrap_line++) {
+    for (; i < len; i++, wrap_line++) {
 	/* record where the last word ended */
 	if (*wrap_line != ' ' && *wrap_line != '\t')
 	    word_back = i;
@@ -1547,8 +1566,10 @@ int do_int_spell_fix(const char *word)
 
     search_last_line = FALSE;
 
-    /* We find the first whole-word occurrence of word. */
-    while (findnextstr(TRUE, TRUE, fileage, -1, word))
+    /* We find the first whole-word occurrence of word.  We call
+       findnextstr() with bracket_mode set to TRUE in order to disable
+       search wrapping. */
+    while (findnextstr(TRUE, TRUE, fileage, -1, word, 0))
 	if (is_whole_word(current_x, current->data, word)) {
 	    edit_refresh();
 
@@ -2094,7 +2115,7 @@ filestruct *backup_lines(filestruct *first_line, size_t par_len,
 
     set_modified();
     cutbuffer = NULL;
-    for(; par_len > 0; par_len--) {
+    for (; par_len > 0; par_len--) {
 	filestruct *bob = copy_node(alice);
 
 	if (alice == first_line)
@@ -2121,7 +2142,7 @@ filestruct *backup_lines(filestruct *first_line, size_t par_len,
 /* Is it possible to break line at or before goal? */
 int breakable(const char *line, int goal)
 {
-    for(; *line != '\0' && goal >= 0; line++) {
+    for (; *line != '\0' && goal >= 0; line++) {
 	if (*line == ' ' || *line == '\t')
 	    return TRUE;
 
@@ -2151,7 +2172,7 @@ int break_line(const char *line, int goal, int force)
 	/* Current index in line */
 
     assert(line != NULL);
-    for(; *line != '\0' && goal >= 0; line++, cur_loc++) {
+    for (; *line != '\0' && goal >= 0; line++, cur_loc++) {
 	if (*line == ' ')
 	    space_loc = cur_loc;
 	assert(*line != '\t');
@@ -2167,7 +2188,7 @@ int break_line(const char *line, int goal, int force)
     if (space_loc == -1) {
 	/* No space found short enough. */
 	if (force)
-	    for(; *line != '\0'; line++, cur_loc++)
+	    for (; *line != '\0'; line++, cur_loc++)
 		if (*line == ' ' && *(line + 1) != ' ' && *(line + 1) != '\0')
 		    return cur_loc;
 	return -1;
@@ -2198,38 +2219,40 @@ int do_para_operation(int operation)
  *   A line is "part of a paragraph" if it has a part not in the quote
  *   part or the indentation.
  *
- *   A line is "the beginning of a paragraph" if it is part of a paragraph
- *   and
+ *   A line is "the beginning of a paragraph" if it is part of a
+ *   paragraph and
  *	1) it is the top line of the file, or
  *	2) the line above it is not part of a paragraph, or
  *	3) the line above it does not have precisely the same quote
  *	   part, or
- *	4) the indentation of this line is not an initial substring of the
- *	   indentation of the previous line, or
+ *	4) the indentation of this line is not an initial substring of
+ *	   the indentation of the previous line, or
  *	5) this line has no quote part and some indentation, and
  *	   AUTOINDENT is not set.
  *   The reason for number 5) is that if AUTOINDENT is not set, then an
- *   indented line is expected to start a paragraph, like in books.  Thus,
- *   nano can justify an indented paragraph only if AUTOINDENT is turned
- *   on.
+ *   indented line is expected to start a paragraph, like in books.
+ *   Thus, nano can justify an indented paragraph only if AUTOINDENT is
+ *   turned on.
  *
  *   A contiguous set of lines is a "paragraph" if each line is part of
- *   a paragraph and only the first line is the beginning of a paragraph.
- */
+ *   a paragraph and only the first line is the beginning of a
+ *   paragraph. */
 
     size_t quote_len;
-	/* Length of the initial quotation of the paragraph we justify. */
+	/* Length of the initial quotation of the paragraph we
+	 * justify. */
     size_t par_len;
 	/* Number of lines in that paragraph. */
     filestruct *first_mod_line = NULL;
 	/* Will be the first line of the resulting justified paragraph
-	 * that differs from the original.  For restoring after uncut. */
+	 * that differs from the original.  For restoring after
+	 * uncut. */
     filestruct *last_par_line = current;
 	/* Will be the last line of the result, also for uncut. */
     filestruct *cutbuffer_save = cutbuffer;
 	/* When the paragraph gets modified, all lines from the changed
-	 * one down are stored in the cut buffer.  We back up the original
-	 * to restore it later. */
+	 * one down are stored in the cut buffer.  We back up the
+	 * original to restore it later. */
 
     /* We save these global variables to be restored if the user
      * unjustifies.  Note we don't need to save totlines. */
@@ -2245,13 +2268,13 @@ int do_para_operation(int operation)
     int mark_beginx_save = mark_beginx;
 #endif
 
-    size_t indent_len;	/* generic indentation length */
-    filestruct *line;	/* generic line of text */
-    size_t i;		/* generic loop variable */
+    size_t indent_len;	/* Generic indentation length. */
+    filestruct *line;	/* Generic line of text. */
+    size_t i;		/* Generic loop variable. */
 
     static int no_restart = 0;
-    	/* whether we're blocking restarting when searching for the
-    	 * beginning line of the paragraph */
+    	/* Whether we're blocking restarting when searching for the
+    	 * beginning line of the paragraph. */
 
 #ifdef HAVE_REGEX_H
     regex_t qreg;	/* qreg is the compiled quotation regexp. 
@@ -2276,15 +2299,15 @@ int do_para_operation(int operation)
 
   restart_bps:
 /* Here we find the first line of the paragraph to justify.  If the
- * current line is in a paragraph, then we move back to the first line. 
+ * current line is in a paragraph, then we move back to the first line.
  * Otherwise we move down to the first line that is in a paragraph. */
     quote_len = quote_length(IFREG(current->data, &qreg));
     indent_len = indent_length(current->data + quote_len);
 
     if (current->data[quote_len + indent_len] != '\0') {
 	/* This line is part of a paragraph.  So we must search back to
-	 * the first line of this paragraph.  First we check items 1) and
-	 * 3) above. */
+	 * the first line of this paragraph.  First we check items 1)
+	 * and 3) above. */
 	while (current->prev != NULL && quotes_match(current->data,
 			quote_len, IFREG(current->prev->data, &qreg))) {
 	    size_t temp_id_len =
@@ -2396,7 +2419,7 @@ int do_para_operation(int operation)
 /* Next step, we loop through the lines of this paragraph, justifying
  * each one individually. */
     SET(JUSTIFY_MODE);
-    for(; par_len > 0; current_y++, par_len--) {
+    for (; par_len > 0; current_y++, par_len--) {
 	size_t line_len;
 	size_t display_len;
 	    /* The width of current in screen columns. */
@@ -2432,8 +2455,8 @@ int do_para_operation(int operation)
 	    if (first_mod_line == NULL)
 		first_mod_line = backup_lines(current, par_len, quote_len);
 	    if (par_len == 1) {
-		/* There is no next line in this paragraph.  We make a new
-		 * line and copy text after break_pos into it. */
+		/* There is no next line in this paragraph.  We make a
+		 * new line and copy text after break_pos into it. */
 		splice_node(current, make_new_node(current),
 				current->next);
 		/* In a non-quoted paragraph, we copy the indent only if
@@ -2490,8 +2513,8 @@ int do_para_operation(int operation)
 
 	    indent_len = quote_len +
 			indent_length(current->next->data + quote_len);
-	    /* If we can't pull a word from the next line up to this one,
-	     * just go on. */
+	    /* If we can't pull a word from the next line up to this
+	     * one, just go on. */
 	    if (!breakable(current->next->data + indent_len,
 		    fill - display_len - 1))
 		goto continue_loc;
@@ -2622,7 +2645,7 @@ int do_para_operation(int operation)
 	edit_refresh();
 
     statusbar(_("Can now UnJustify!"));
-    /* Change the shortcut list to display the unjustify code */
+    /* Change the shortcut list to display the unjustify code. */
     shortcut_init(1);
     display_main_list();
     reset_cursor();
@@ -2630,16 +2653,22 @@ int do_para_operation(int operation)
     /* Now get a keystroke and see if it's unjustify; if not, unget the
      * keystroke and return. */
 
+    {
+	int meta;
+	i = get_kbinput(edit, &meta);
 #ifndef DISABLE_MOUSE
-    /* If it was a mouse click, parse it with do_mouse() and it might
-     * become the unjustify key.  Else give it back to the input stream. */
-    if ((i = wgetch(edit)) == KEY_MOUSE)
-	do_mouse();
-    else
-	ungetch(i);
+	/* If it was a mouse click, parse it with do_mouse() and it
+	 * might become the unjustify key.  Else give it back to the
+	 * input stream. */
+	if (i == KEY_MOUSE)
+	    do_mouse();
+	else
+	    ungetch(i);
+	i = get_kbinput(edit, &meta);
 #endif
+    }
 
-    if ((i = wgetch(edit)) != NANO_UNJUSTIFY_KEY) {
+    if (i != NANO_UNJUSTIFY_KEY) {
 	ungetch(i);
 	/* Did we back up anything at all? */
 	if (cutbuffer != cutbuffer_save)
@@ -2734,14 +2763,11 @@ int do_exit(void)
 	    finish(0);
     }
 
-    if (ISSET(TEMP_OPT)) {
+    if (ISSET(TEMP_OPT))
 	i = 1;
-    } else {
-	i = do_yesno(0, 0,
-		     _
-		     ("Save modified buffer (ANSWERING \"No\" WILL DESTROY CHANGES) ? "));
-    }
-
+    else
+	i = do_yesno(0, _("Save modified buffer (ANSWERING \"No\" WILL DESTROY CHANGES) ? "));
+    
 #ifdef DEBUG
     dump_buffer(fileage);
 #endif
@@ -2872,7 +2898,7 @@ RETSIGTYPE do_cont(int signal)
     doupdate();
 
     /* The Hurd seems to need this, otherwise a ^Y after a ^Z will
-	start suspending again. */
+       start suspending again. */
     signal_init();
 
 #ifndef NANO_SMALL
@@ -3029,7 +3055,7 @@ int main(int argc, char *argv[])
     int fill_flag_used = 0;	/* Was the fill option used? */
     const shortcut *s;
     int keyhandled = 0;	/* Have we handled the keystroke yet? */
-    int kbinput = -1;		/* Input from keyboard */
+    int kbinput;		/* Input from keyboard */
     int meta;
 
 #ifndef NANO_SMALL
@@ -3039,7 +3065,6 @@ int main(int argc, char *argv[])
     struct termios term;
 #endif
 #ifdef HAVE_GETOPT_LONG
-    int option_index = 0;
     const struct option long_options[] = {
 	{"help", 0, 0, 'h'},
 #ifdef ENABLE_MULTIBUFFER
@@ -3113,7 +3138,7 @@ int main(int argc, char *argv[])
 
 #ifdef HAVE_GETOPT_LONG
     while ((optchr = getopt_long(argc, argv, "h?BDFHIMNQ:RST:VY:abcdefgijklmo:pr:s:tvwxz",
-				 long_options, &option_index)) != -1) {
+				 long_options, NULL)) != -1) {
 #else
     while ((optchr =
 	    getopt(argc, argv, "h?BDFHIMNQ:RST:VY:abcdefgijklmo:pr:s:tvwxz")) != -1) {
@@ -3488,70 +3513,15 @@ int main(int argc, char *argv[])
 	raw();
 #endif
 
-	kbinput = get_kbinput(edit, &meta, ISSET(REBIND_DELETE));
+	kbinput = get_kbinput(edit, &meta);
 #ifdef DEBUG
 	fprintf(stderr, "AHA!  %c (%d)\n", kbinput, kbinput);
 #endif
 	if (meta == 1) {
-	    switch (kbinput) {
-#ifdef ENABLE_MULTIBUFFER
-	    case NANO_OPENPREV_KEY:
-	    case NANO_OPENPREV_ALTKEY:
-		open_prevfile_void();
-		keyhandled = 1;
-		break;
-	    case NANO_OPENNEXT_KEY:
-	    case NANO_OPENNEXT_ALTKEY:
-		open_nextfile_void();
-		keyhandled = 1;
-		break;
-#endif
-	    default:
-		/* Check for the altkey defs.... */
-		for (s = main_list; s != NULL; s = s->next)
-		    if (kbinput == s->altval || (kbinput >= 'A' &&
-			    kbinput <= 'Z' && kbinput == s->altval - 32)) {
-			if (ISSET(VIEW_MODE) && !s->viewok)
-			    print_view_warning();
-			else {
-			    if (s->func != do_cut_text)
-				UNSET(KEEP_CUTBUFFER);
-			    s->func();
-			}
-			keyhandled = 1;
-			break;
-		    }
-#ifndef NANO_SMALL
-		if (!keyhandled)
-		    /* And for toggle switches */
-		    for (t = toggles; t != NULL; t = t->next)
-			if (kbinput == t->val || (t->val >= 'a' &&
-				t->val <= 'z' && kbinput == t->val - 32)) {
-			    UNSET(KEEP_CUTBUFFER);
-			    do_toggle(t);
-			    keyhandled = 1;
-			    break;
-		        }
-#endif
-#ifdef DEBUG
-		fprintf(stderr, "I got Alt-%c! (%d)\n", kbinput,
-			kbinput);
-#endif
-	    }
-	}
-
-	/* Look through the main shortcut list to see if we've hit a
-	   shortcut key */
-
-	if (!keyhandled)
-#if !defined(DISABLE_BROWSER) || !defined (DISABLE_HELP) || !defined(DISABLE_MOUSE)
-	    for (s = currshortcut; s != NULL && !keyhandled; s = s->next) {
-#else
-	    for (s = main_list; s != NULL && !keyhandled; s = s->next) {
-#endif
-		if (kbinput == s->val ||
-		    (s->misc1 && kbinput == s->misc1) ||
-		    (s->misc2 && kbinput == s->misc2)) {
+	    /* Check for the metaval and misc defs... */
+	    for (s = main_list; s != NULL; s = s->next)
+		if ((s->metaval != NANO_NO_KEY && kbinput == s->metaval) ||
+		    (s->misc != NANO_NO_KEY && kbinput == s->misc)) {
 		    if (ISSET(VIEW_MODE) && !s->viewok)
 			print_view_warning();
 		    else {
@@ -3560,10 +3530,44 @@ int main(int argc, char *argv[])
 			s->func();
 		    }
 		    keyhandled = 1;
-		    /* Rarely, the value of s can change after
-		       s->func(), leading to problems; get around this
-		       by breaking out explicitly once we successfully
-		       handle a shortcut */
+		}
+#ifndef NANO_SMALL
+		if (!keyhandled)
+		    /* And for toggle switches */
+		    for (t = toggles; t != NULL; t = t->next)
+			if (kbinput == t->val) {
+			    UNSET(KEEP_CUTBUFFER);
+			    do_toggle(t);
+			    keyhandled = 1;
+		        }
+#endif
+#ifdef DEBUG
+	    fprintf(stderr, "I got Alt-%c! (%d)\n", kbinput,
+		kbinput);
+#endif
+	}
+
+	/* Look through the main shortcut list to see if we've hit a
+	   shortcut key or function key */
+
+	if (!keyhandled)
+#if !defined(DISABLE_BROWSER) || !defined (DISABLE_HELP) || !defined(DISABLE_MOUSE)
+	    for (s = currshortcut; s != NULL && !keyhandled; s = s->next) {
+#else
+	    for (s = main_list; s != NULL && !keyhandled; s = s->next) {
+#endif
+		if ((s->val != NANO_NO_KEY && kbinput == s->val) ||
+		    (s->func_key != NANO_NO_KEY && kbinput == s->func_key)) {
+		    if (ISSET(VIEW_MODE) && !s->viewok)
+			print_view_warning();
+		    else {
+			if (s->func != do_cut_text)
+			    UNSET(KEEP_CUTBUFFER);
+			s->func();
+		    }
+		    keyhandled = 1;
+		    /* Break out explicitly once we successfully handle
+		       a shortcut */
 		    break;
 		}
 	    }
