@@ -1,8 +1,8 @@
-/* $Id: utils.c,v 1.2 2003/09/16 01:16:49 dolorous Exp $ */
+/* $Id: utils.c,v 1.5 2004/01/09 23:04:55 dolorous Exp $ */
 /**************************************************************************
  *   utils.c                                                              *
  *                                                                        *
- *   Copyright (C) 1999-2003 Chris Allegretta                             *
+ *   Copyright (C) 1999-2004 Chris Allegretta                             *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -84,7 +84,7 @@ void null_at(char **data, size_t index)
 void unsunder(char *str, size_t true_len)
 {
     assert(str != NULL);
-    for(; true_len > 0; true_len--, str++)
+    for (; true_len > 0; true_len--, str++)
 	if (*str == '\0')
 	    *str = '\n';
 }
@@ -94,7 +94,7 @@ void unsunder(char *str, size_t true_len)
 void sunder(char *str)
 {
     assert(str != NULL);
-    for(; *str != '\0'; str++)
+    for (; *str != '\0'; str++)
 	if (*str == '\n')
 	    *str = '\0';
 }
@@ -135,7 +135,7 @@ int nstrnicmp(const char *s1, const char *s2, size_t n)
 const char *revstrstr(const char *haystack, const char *needle,
 			const char *rev_start)
 {
-    for(; rev_start >= haystack ; rev_start--) {
+    for (; rev_start >= haystack; rev_start--) {
 	const char *r, *q;
 
 	for (r = rev_start, q = needle ; *q == *r && *q != '\0'; r++, q++)
@@ -197,13 +197,13 @@ const char *strstrwrapper(const char *haystack, const char *needle,
 #ifndef NANO_SMALL
 	if (ISSET(REVERSE_SEARCH)) {
 		/* When doing a backwards search, haystack is a whole line. */
-	    if (!regexec(&search_regexp, haystack, 1, regmatches, 0) &&
+	    if (regexec(&search_regexp, haystack, 1, regmatches, 0) == 0 &&
 		    haystack + regmatches[0].rm_so <= rev_start) {
 		const char *retval = haystack + regmatches[0].rm_so;
 
 		/* Search forward until there is no more match. */
-		while (!regexec(&search_regexp, retval + 1, 1, regmatches,
-			    REG_NOTBOL) &&
+		while (regexec(&search_regexp, retval + 1, 1, regmatches,
+			    REG_NOTBOL) == 0 &&
 			retval + 1 + regmatches[0].rm_so <= rev_start)
 		    retval += 1 + regmatches[0].rm_so;
 		/* Finally, put the subexpression matches in global
@@ -214,8 +214,8 @@ const char *strstrwrapper(const char *haystack, const char *needle,
 	    }
 	} else
 #endif /* !NANO_SMALL */
-	if (!regexec(&search_regexp, haystack, 10, regmatches,
-			line_pos > 0 ? REG_NOTBOL : 0)) {
+	if (regexec(&search_regexp, haystack, 10, regmatches,
+			line_pos > 0 ? REG_NOTBOL : 0) == 0) {
 	    const char *retval = haystack + regmatches[0].rm_so;
 
 	    regexec(&search_regexp, retval, 10, regmatches, 0);
